<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Entities;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\{BorderType, Height, Id, Name, Style, Width};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for an icon.
 *
 * @property-read BorderType $border_type The border type.
 * @property-read Height $height The height.
 * @property-read Id $id The id.
 * @property-read Name $name The name.
 * @property-read Style $style The style.
 * @property-read Width $width The width.
 */
final class OnlineMap {
	/**
	 * The constructor.
	 *
	 * @param Id         $id   The id.
	 * @param Name       $name The name.
	 * @param Style      $style The style.
	 * @param Width      $width The width.
	 * @param Height     $height The height.
	 * @param BorderType $border_type The border type.
	 */
	public function __construct(
		private Id $id,
		private Name $name = new Name( '' ),
		private Style $style = new Style(),
		private Width $width = new Width(),
		private Height $height = new Height(),
		private BorderType $border_type = BorderType::None,
	) {
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @return mixed
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'border_type' => $this->border_type,
			'height' => $this->height,
			'id' => $this->id,
			'name' => $this->name,
			'style' => $this->style,
			'width' => $this->width,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * Builds the border json.
	 */
	public function build_border_json(): string {
		$border_data = array(
			'type' => $this->border_type->value,
		);

		$json = wp_json_encode( $border_data );
		if ( false === $json ) {
			$json = '{}';
		}

		return $json;
	}

	/**
	 * Builds an icon instance from a database row.
	 *
	 * @param mixed $row The database row.
	 * @throws \InvalidArgumentException If the "id"-field is missing.
	 */
	public static function from_db_row( mixed $row ): self {
		if ( ! is_array( $row ) ) {
			throw new \InvalidArgumentException( 'The row must be an array.' );
		}
		if ( ! isset( $row['id'] ) ) {
			throw new \InvalidArgumentException( 'The "id"-field is missing from the database.' );
		}
		// name can be null!
		if ( ! array_key_exists( 'name', $row ) ) {
			throw new \InvalidArgumentException( 'The "name"-field is missing from the database.' );
		}
		if ( ! isset( $row['style'] ) ) {
			throw new \InvalidArgumentException( 'The "style"-field is missing from the database.' );
		}
		if ( ! isset( $row['width'] ) ) {
			throw new \InvalidArgumentException( 'The "width"-field is missing from the database.' );
		}
		if ( ! isset( $row['height'] ) ) {
			throw new \InvalidArgumentException( 'The "height"-field is missing from the database.' );
		}

		$border_type = BorderType::None;
		if ( isset( $row['border'] ) && is_string( $row['border'] ) ) {
			$border_data = json_decode( $row['border'], true );
			if ( is_array( $border_data ) && isset( $border_data['type'] ) && is_string( $border_data['type'] ) ) {
				$border_type = BorderType::tryFrom( $border_data['type'] ) ?? BorderType::None;
			}
		}

		return new self(
			id: new Id( $row['id'] ),
			name: new Name( $row['name'] ?? '' ),
			style: new Style( $row['style'] ),
			width: new Width( $row['width'] ),
			height: new Height( $row['height'] ),
			border_type: $border_type,
		);
	}

	/**
	 * Converts the icon to an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			'id'     => $this->id->value,
			'name'   => (string) $this->name,
			'style'  => (string) $this->style,
			'width'  => (string) $this->width,
			'height' => (string) $this->height,
			'border' => array(
				'type' => $this->border_type->value,
			),
		);
	}

	/**
	 * Updates the id.
	 *
	 * @param Id $id The id.
	 */
	public function update_id( Id $id ): self {
		$this->id = $id;
		return $this;
	}

	/**
	 * Updates the name.
	 *
	 * @param Name $name The name.
	 */
	public function update_name( Name $name ): self {
		$this->name = $name;
		return $this;
	}

	/**
	 * Updates the style.
	 *
	 * @param Style $style The style.
	 */
	public function update_style( Style $style ): self {
		$this->style = $style;
		return $this;
	}

	/**
	 * Updates the width.
	 *
	 * @param Width $width The width.
	 */
	public function update_width( Width $width ): self {
		$this->width = $width;
		return $this;
	}

	/**
	 * Updates the height.
	 *
	 * @param Height $height The height.
	 */
	public function update_height( Height $height ): self {
		$this->height = $height;
		return $this;
	}

	/**
	 * Updates the border type.
	 *
	 * @param BorderType $border_type The border type.
	 */
	public function update_border_type( BorderType $border_type ): self {
		$this->border_type = $border_type;
		return $this;
	}
}
